;(function() {
	var sdkName="HighFive"
	if (typeof window[sdkName] == "object") {return}
	function run(fn) {
		if (fn instanceof Function) {return fn.apply(null, [].slice.call(arguments).splice(1))}
	}
	var ErrUnsupport = "unsupported_action"
	var ErrInvalidOP = "invalid_operation"
	var sdk = {
		_completions:{},_environments:{},_handlers:{},
		setHandler: function(name, fn) {
			this._handlers[name] = fn instanceof Function ? fn : null
		},
		_sessionEnded: function() {run(this._handlers.sessionEnded)},
		_paused: function() {
			this._environments.paused = true
			run(this._handlers.paused)
		},
		_resumed: function() {
			this._environments.paused = false
			run(this._handlers.resumed)
		},
	
		_closeButtonTouchUp: function() {
			var fn = this._handlers.closing
			this.invoke("closeEventDidReceive", fn instanceof Function ? "handled" : "no_handler")
			run(fn)
		},
	
		_invokeDidFinish: function(action, args) {
			var fn = this._completions[action]
			if (fn instanceof Function) {
				fn(typeof args == "object" ? args : new Error(args))
				delete this._completions[action]
			}
		},
	
		mergeEnvironments: function(args) {
			if (args instanceof Object) {
				for (var key in args) {this._environments[key] = args[key]}
			}
		},
	
		setCompletionForAction: function(action, fn) {
			if (!action || !(fn instanceof Function)) {return}
			this._completions[action] = fn
		},
	
		invoke: function(action, args, comp) {
			var bridge = window.H5GameBridge
			if (!bridge) {
				run(comp, new Error("platform_invalid"))
				return
			}
			this.setCompletionForAction(action, comp)
			args = args || {}
			console.log(["invoke", action, args])
			bridge.invoke(action, typeof args == "object" ? JSON.stringify(args) : args)
		},
	
		addActions: function(acts /*[API]*/, name) {
			var comp = this
			var base = ""
			if (name) {
				comp[name] = comp[name] || {}
				comp = comp[name]
				base = name + "."
			}
			var self = this
			acts.forEach(function(action) {
				action.api = base + action.name
				comp[action.name] = action.invocation
			})
		},
	
		getSupportedAPIs: function() {return this._environments.supported_apis},
		isAPISupported: function(api) {
			var apis = this.getSupportedAPIs()
			return apis instanceof Array && apis.indexOf(api) >= 0
		},
		isGDPRGranted: function() {
			var v = this._environments.gdpr_granted
			return typeof v == "boolean" ? v : true
		},
		getLocale: function() {return this._environments.locale},
		getPlatform: function() {return this._environments.platform},
		getStartupDuration: function() {return parseInt(this._environments.startup_dur) || 0},
	
		getExchangeRate: function() {return parseFloat(this._environments.exchange_rate) || 1},
		getCoin: function() {return parseFloat(this._environments.coin) || 0},
		getExchangedCoin: function() {return this.getCoin() * this.getExchangeRate()},
		setExchangedCoin: function(coin) {
			this._environments.coin = coin / this.getExchangeRate()
			this.invoke("setCoin", this._environments.coin)
		},
		isPaused: function() {return !!this._environments.paused},
		onPause: function(fn) {this.setHandler("paused", fn)},
		onResume: function(fn) {this.setHandler("resumed", fn)},
		onClose: function(fn) {this.setHandler("closing", fn)},
		onSessionEnd: function(fn) {this.setHandler("sessionEnded", fn)},
	}
	sdk.player = {
		getID: function() {return sdk._environments.player_id},
		getName: function() {return sdk._environments.player_name},
		getPhoto: function() {return sdk._environments.player_photo},
	}
	sdk.context = {
		getID: function() {return sdk._environments.context_id},
		getType: function() {return sdk._environments.context_type},
	}
	sdk.AdType = {interstitial: "InterstitialAd", rewardedVideo: "RewardedVideo"}
	window[sdkName]=sdk
	
	function Ad(type) {
		this.type = type
	}
	Ad.prototype.loadAsync = function() {
		var self = this
		self.state = "loading"
		var api = "ad.load" + this.type + "Async"
		return new Promise(function(resolve, reject) {
			sdk.invoke(api, null, function(args) {
				if (args instanceof Error) {
					self.state = "load_failure"
					run(reject, args)
				} else {
					self.state = "loaded"
					run(resolve)
				}
			})
		})
	}
	Ad.prototype.showAsync = function() {
		var self = this
		self.state = "showing"
		var api = "ad.show" + this.type + "Async"
		return new Promise(function(resolve, reject) {
			sdk.invoke(api, null, function(args) {
				self.state = "shown"
				if (args instanceof Error) {run(reject, args)}
				else {run(resolve, args)}
			})
		})
	}
	
	function API(name, fn) {
		this.name = name
		this.invocation = fn.bind(this)
	}
	
	sdk.addActions([
		new API("initializeAsync", function() {
			var api = this.api
			return new Promise(function(resolve, reject) {
				if (sdk.player.getID()) {
					run(reject, new Error(ErrInvalidOP))
					return
				}
				sdk.invoke(api, {handler: sdkName}, function(args) {
					if (args instanceof Error) {run(reject, args)}
					else {
						sdk.mergeEnvironments(args)
						run(resolve)
					}
				})
			})
		}),
	
		new API("startGameAsync", function() {
			var api = this.api
			return new Promise(function(resolve, reject) {
				sdk.invoke(api, null, function(args) {
					if (args instanceof Error) {run(reject, args)}
					else {
						sdk.mergeEnvironments(args)
						run(resolve)
					}
				})
			})
		}),
	
		new API("quit", function() {
			sdk.invoke(this.api)
		}),

	    new API("goBack", function () {
                sdk.invoke(this.api)
            }),
		new API("logEvent", function(event, params) {
			sdk.invoke(this.api, {event: event, parameters: params})
		}),
		new API("setUserLevel", function(level) {
			if (typeof level != "number") {
				level = parseFloat(level)
			}
			if (isNaN(level)) {return}
			sdk.invoke(this.api, {level: level})
		}),
	
		new API("getInterstitialAdAsync", function() {
			var api = this.api
			return new Promise(function(resolve, reject) {
				sdk.invoke(api, null, function(args) {
					if (args instanceof Error) {run(reject, args)}
					else {run(resolve, new Ad("InterstitialAd"))}
				})
			})
		}),
	
		new API("getRewardedVideoAsync", function() {
			var api = this.api
			return new Promise(function(resolve, reject) {
				if (!sdk.isAPISupported(api)) {
					run(reject, new Error(ErrUnsupport))
					return
				}
				sdk.invoke(api, null, function(args) {
					if (args instanceof Error) {run(reject, args)}
					else {run(resolve, new Ad("RewardedVideo"))}
				})
			})
		}),
	
		new API("adShowChanceArrived", function(shown, type) {
			sdk.invoke(this.api, {shown: !!shown ? "shown" : "", type: type})
		}),
	])
	
	sdk.addActions([
		new API("getDataAsync", function(names) {
			var api = this.api
			return new Promise(function(resolve, reject) {
				sdk.invoke(api, names, resolve)
			})
		}),
		new API("setDataAsync", function(data) {
			var api = this.api
			return new Promise(function(resolve, reject) {
				sdk.invoke(api, data, resolve)
			})
		}),
		new API("setStatsAsync", function(data) {
			var api = this.api
			return new Promise(function(resolve, reject) {
				sdk.invoke(api, data, resolve)
			})
		}),
	], "player")
	})()
	